@@ -514,11 +514,7 @@ int dbOSFile::read(offs_t pos, void* buf, size_t size)
 #else
     ssize_t rc;
 #if defined(__sun) || defined(_AIX43) || defined(__linux__)
-#if !defined(SOLARIS) || defined(L64)
-    rc = pread64(fd, buf, size, pos);
-#else
     rc = pread(fd, buf, size, pos);
-#endif
 #else
     {
         dbCriticalSection cs(mutex);
@@ -600,11 +596,7 @@ int dbOSFile::write(offs_t pos, void const* buf, size_t size)
 #else
     ssize_t rc;
 #if defined(__sun) || defined(_AIX43) || defined(__linux__)
-#if !defined(SOLARIS) || defined(L64)
-    rc = pwrite64(fd, buf, size, pos);
-#else 
     rc = pwrite(fd, buf, size, pos);
-#endif
 #else
     {
         dbCriticalSection cs(mutex);
@@ -675,6 +667,9 @@ BEGIN_GIGABASE_NAMESPACE
 
 size_t dbOSFile::ramSize()
 {
+#ifdef _SC_PHYS_PAGES
+    return sysconf(_SC_PHYS_PAGES)*sysconf(_SC_PAGE_SIZE);
+#else
     struct sysinfo info;
     sysinfo(&info);
 #ifdef SYSINFO_HAS_NO_MEM_UNIT
@@ -682,6 +677,7 @@ size_t dbOSFile::ramSize()
 #else
     return info.totalram*info.mem_unit;
 #endif
+#endif
 }
 #else
 size_t dbOSFile::ramSize()
