@@ -1,4 +1,4 @@
-//-< BTREE.CPP >-----------------------------------------------------*--------*
+//-< BTREE.H >-------------------------------------------------------*--------*
 // GigaBASE                  Version 1.0         (c) 1999  GARRET    *     ?  *
 // (Post Relational Database Management System)                      *   /\|  *
 //                                                                   *  /  \  *
@@ -14,6 +14,10 @@
 
 BEGIN_GIGABASE_NAMESPACE
 
+class dbAnyCursor;
+
+#define MAX_BTREE_HEIGHT 8
+
 class dbBtreePage {
   public:
     nat4 nItems;
@@ -62,7 +66,7 @@ class dbBtreePage {
     static oid_t allocate(dbDatabase* db, oid_t root, int type, int sizeofType, item& ins);
 
     static int   insert(dbDatabase* db, oid_t pageId,
-                        int type, int sizeofType, dbUDTComparator comparator, item& ins, int height);
+                        int type, int sizeofType, dbUDTComparator comparator, item& ins, int height, bool unique);
     static int   remove(dbDatabase* db, oid_t pageId,
                         int type, int sizeofType, dbUDTComparator comparator, item& rem, int height);
 
@@ -70,8 +74,8 @@ class dbBtreePage {
 
     int          insertStrKey(dbDatabase* db, int r, item& ins, int height);
     int          replaceStrKey(dbDatabase* db, int r, item& ins, int height);
-    int          removeStrKey(int r);
-    void         compactify(int m);
+    int          removeStrKey(dbDatabase* db, int r);
+    void         compactify(dbDatabase* db, int m);
 
     int          handlePageUnderflow(dbDatabase* db, int r, int type, int sizeofType, item& rem,
                                      int height);
@@ -87,6 +91,7 @@ class dbBtreePage {
 
 
 class GIGABASE_DLL_ENTRY dbBtree : public dbRecord {
+    friend class dbBtreeIterator;
   protected:
     oid_t root;
     int4  height;
@@ -96,7 +101,8 @@ class GIGABASE_DLL_ENTRY dbBtree : public dbRecord {
   public:
     enum { 
         FLAGS_CASE_INSENSITIVE = 1, 
-        FLAGS_THICK = 2
+        FLAGS_THICK = 2,
+        FLAGS_UNIQUE = 4
     };
     int1  flags; 
 
@@ -104,17 +110,28 @@ class GIGABASE_DLL_ENTRY dbBtree : public dbRecord {
         return (flags & FLAGS_CASE_INSENSITIVE) != 0;
     }
 
+    bool isThick() { 
+        return (flags & FLAGS_THICK) != 0;
+    }
+
+    bool isUnique() { 
+        return (flags & FLAGS_UNIQUE) != 0;
+    }
+
     enum OperationEffect {
         done,
         overflow,
         underflow,
-        not_found
+        not_found,                              
+        not_unique
     };
 
     static oid_t allocate(dbDatabase* db, int type, int sizeofType, int flags = 0);
     static void  find(dbDatabase* db, oid_t treeId, dbSearchContext& sc, dbUDTComparator comparator);
-    static void  insert(dbDatabase* db, oid_t treeId, oid_t recordId, int offs, dbUDTComparator comparator);
-    static void  insert(dbDatabase* db, oid_t treeId, dbBtreePage::item& ins, dbUDTComparator comparator);
+    static bool  insert(dbDatabase* db, oid_t treeId, oid_t recordId, byte* record, int offs, dbUDTComparator comparator);
+    static bool  insert(dbDatabase* db, oid_t treeId, oid_t recordId, int offs, dbUDTComparator comparator);
+    static bool  insert(dbDatabase* db, oid_t treeId, dbBtreePage::item& ins, dbUDTComparator comparator);
+    static void  remove(dbDatabase* db, oid_t treeId, oid_t recordId, byte* record, int offs, dbUDTComparator comparator);
     static void  remove(dbDatabase* db, oid_t treeId, oid_t recordId, int offs, dbUDTComparator comparator);
     static void  drop(dbDatabase* db, oid_t treeId);
     static void  purge(dbDatabase* db, oid_t treeId);
@@ -135,7 +152,45 @@ class GIGABASE_DLL_ENTRY dbBtree : public dbRecord {
     }
 };
 
+class GIGABASE_DLL_ENTRY dbBtreeIterator : public dbAbstractIterator { 
+  public:
+    void init(dbDatabase* db, oid_t treeId, dbSearchContext& sc, dbUDTComparator comparator);
+    
+    oid_t next();
+    oid_t prev();
+    oid_t first();
+    oid_t last();
+    
+  private:
+    oid_t reset(bool ascent); 
+    oid_t gotoNextItem(byte* pg, int pos, bool forward);
+    
+    oid_t getStringBtreePageOid(byte* pg, int i);
+    oid_t getScalarBtreePageOid(byte* pg, int i);
+    oid_t getStringThickBtreePageOid(byte* pg, int i);
+    oid_t getScalarThickBtreePageOid(byte* pg, int i);
+    void* getStringBtreePageKey(byte* pg, int i);
+    void* getScalarBtreePageKey(byte* pg, int i);
+    void* getStringThickBtreePageKey(byte* pg, int i);
+    void* getScalarThickBtreePageKey(byte* pg, int i);
+    
+    oid_t (dbBtreeIterator::*getOid)(byte* pg, int i);
+    void* (dbBtreeIterator::*getKey)(byte* pg, int i);
+    
+    int nItems(byte* pg) { 
+        return ((dbBtreePage*)pg)->nItems;
+    }
 
+    dbUDTComparator comparator;
+    dbDatabase*     db;
+    dbSearchContext sc;
+    int             sizeofType;
+    int             type;
+    int             height;
+    oid_t           treeId;
+    oid_t           pageStack[MAX_BTREE_HEIGHT];
+    int             posStack[MAX_BTREE_HEIGHT];
+};   
 
 class dbThickBtreePage {
   public:
@@ -196,8 +251,8 @@ class dbThickBtreePage {
 
     int          insertStrKey(dbDatabase* db, int r, item& ins, int height);
     int          replaceStrKey(dbDatabase* db, int r, item& ins, int height);
-    int          removeStrKey(int r);
-    void         compactify(int m);
+    int          removeStrKey(dbDatabase* db, int r);
+    void         compactify(dbDatabase* db, int m);
 
     int          handlePageUnderflow(dbDatabase* db, int r, int type, int sizeofType, item& rem,
                                      int height);
