@@ -1297,7 +1297,7 @@ int cli_remove_current(int statement)
 
 int cli_describe(int session, char_t const* table, cli_field_descriptor** fields)
 {
-    int len = (int)(sizeof(cli_request) + STRLEN(table)*sizeof(char_t) + 1);
+    int len = (int)(sizeof(cli_request) + (STRLEN(table)+1)*sizeof(char_t));
     dbSmallBuffer<char> buf(len);
     cli_request* req = (cli_request*)buf.base();
     req->length = len;
@@ -1631,3 +1631,7 @@ int cli_get_database_size(int session, cli_nat8_t* size)
     return cli_not_implemented;
 }
 
+int cli_get_wrapping_rectangle(int session, char_t const* table, char_t const* field, cli_rectangle_t* rect)
+{
+    return cli_not_implemented;
+}
