@@ -673,7 +673,7 @@ class gb_connection {
     * Returns:
     *     status code
     */
-    function commit() {
+    function precommit() {
 	if (!$this->opened) { 
 	    return cli_closed_connection;
 	}
@@ -1133,7 +1133,7 @@ class gb_statement {
         if ($rc < 0) { 
             return $rc;
         }
-        while ($this->get_next() == cli_ok$rc > 0) { 
+        while ($this->get_next() == cli_ok) { 
             $arr[] = $this->record;
         }
         $this->mode = $save_mode;
