@@ -209,7 +209,7 @@ public class Connection {
         Type c;
         int nColumns = 0;
         for (c = cls; c != null; c = c.BaseType) { 
-            FieldInfo[] classFields = c.GetFields();
+            FieldInfo[] classFields = c.GetFields(BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.DeclaredOnly);
             for (i = 0; i < classFields.Length; i++) { 
                 if (!classFields[i].IsStatic && !classFields[i].IsNotSerialized) { 
                     nColumns += 1;
@@ -221,7 +221,7 @@ public class Connection {
 	buf.putAsciiz(tableName);
         buf.putByte(nColumns);
         for (c = cls; c != null; c = c.BaseType) { 
-            FieldInfo[] classFields = c.GetFields();
+            FieldInfo[] classFields = c.GetFields(BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.DeclaredOnly);
             for (i = 0; i < classFields.Length; i++) { 
 	        FieldInfo f = classFields[i];
                 String refTableName = "";
@@ -344,7 +344,7 @@ public class Connection {
     ///
     public void dropTable(String tableName) 
     { 
-	ComBuffer buf = new ComBuffer(CLICommand.cli_cmd_create_table);
+	ComBuffer buf = new ComBuffer(CLICommand.cli_cmd_drop_table);
 	buf.putAsciiz(tableName);
 	sendReceive(buf);
     }
@@ -412,10 +412,18 @@ public class Connection {
         return rc;
     }
         
+    const int MaxSendParcelSize = 64*1024;
 
     internal void send(ComBuffer com) { 
+        int rc;
         com.end();
-        socket.Send(com.buf, 0, com.pos, SocketFlags.None);
+        for (int size = com.pos, offs = 0; size != 0; size -= rc, offs += rc) { 
+            int chunk = size > MaxSendParcelSize ? MaxSendParcelSize : size;            
+            rc = socket.Send(com.buf, offs, chunk, SocketFlags.None);
+            if (rc <= 0) { 
+                throw new CliError("Socket write failed");
+            }
+        }                
     }
 
     internal void receive(ComBuffer com, int len) { 
@@ -462,6 +470,7 @@ public class Connection {
         cli_cmd_commit,
         cli_cmd_update,
         cli_cmd_remove,
+        cli_cmd_remove_current,
         cli_cmd_insert,
         cli_cmd_prepare_and_insert,
         cli_cmd_describe_table,
@@ -471,7 +480,11 @@ public class Connection {
         cli_cmd_skip,
         cli_cmd_create_table,
         cli_cmd_drop_table, 
-        cli_cmd_alter_index
+        cli_cmd_alter_index,
+        cli_cmd_freeze,
+        cli_cmd_unfreeze,
+        cli_cmd_seek,
+        cli_cmd_alter_table,
     }
     
     /* status codes */
