@@ -8,16 +8,20 @@
 // Hash table is not supported in this version of Gigabase
 //-------------------------------------------------------------------*--------*
 
+#define INSIDE_GIGABASE
+
 #include "gigabase.h"
 #include "hashtab.h"
 
+BEGIN_GIGABASE_NAMESPACE
+
 int const dbHashTable::keySize[] = {
     1,  // tpBool
     1,  // tpInt1
     2,  // tpInt2
     4,  // tpInt4
     8,  // tpInt8
-    4,  // tpReal4 
+    4,  // tpReal4
     8,  // tpReal8
     0,  // tpString,
     -1, // tpReference
@@ -37,7 +41,7 @@ void dbHashTable::insert(dbDatabase*, oid_t, oid_t, int, int, size_t)
 void dbHashTable::remove(dbDatabase*, oid_t, oid_t, int, int)
 {
 }
-    
+
 void dbHashTable::find(dbDatabase*, oid_t, dbSearchContext&)
 {
 }
@@ -53,6 +57,8 @@ void dbHashTable::drop(dbDatabase*, oid_t)
 }
 
 
+END_GIGABASE_NAMESPACE
+
 
 
 
