@@ -26,7 +26,7 @@ public class Connection {
     *  @param hostAddress string with server host name
     *  @param hostPort    integer number with server port
     *  @param userName    user name
-    *  @param password    password
+    *  @param userPassword    password
     */    
     public void open(String hostAddress, int hostPort, String userName, String userPassword) { 
 	address = hostAddress;
@@ -474,16 +474,21 @@ public class Connection {
     protected static final int cli_cmd_commit = 9;
     protected static final int cli_cmd_update = 10;
     protected static final int cli_cmd_remove = 11;
-    protected static final int cli_cmd_insert = 12;
-    protected static final int cli_cmd_prepare_and_insert = 13;
-    protected static final int cli_cmd_describe_table = 14;
-    protected static final int cli_cmd_show_tables = 15;
-    protected static final int cli_cmd_login = 16;
-    protected static final int cli_cmd_precommit = 17;
-    protected static final int cli_cmd_skip = 18;
-    protected static final int cli_cmd_create_table = 19;
-    protected static final int cli_cmd_drop_table = 20;
-    protected static final int cli_cmd_alter_index = 21;
+    protected static final int cli_cmd_remove_current = 12;
+    protected static final int cli_cmd_insert = 13;
+    protected static final int cli_cmd_prepare_and_insert = 14;
+    protected static final int cli_cmd_describe_table = 15;
+    protected static final int cli_cmd_show_tables = 16;
+    protected static final int cli_cmd_login = 17;
+    protected static final int cli_cmd_precommit = 18;
+    protected static final int cli_cmd_skip = 19;
+    protected static final int cli_cmd_create_table = 20;
+    protected static final int cli_cmd_drop_table = 21;
+    protected static final int cli_cmd_alter_index = 22;
+    protected static final int cli_cmd_freeze = 23;
+    protected static final int cli_cmd_unfreeze = 24;
+    protected static final int cli_cmd_seek = 25;
+    protected static final int cli_cmd_alter_table = 26;
     
     /* status codes */
     public static final int cli_ok = 0;
