@@ -18,9 +18,23 @@ BEGIN_GIGABASE_NAMESPACE
 #define DEFAULT_CONNECT_MAX_ATTEMPTS 100
 #define DEFAULT_RECONNECT_TIMEOUT    1
 #define DEFAULT_LISTEN_QUEUE_SIZE    5
-#define LINGER_TIME                  10
 #define WAIT_FOREVER                 ((time_t)-1)
 
+#ifndef LINGER_TIME
+#define LINGER_TIME                  10
+#endif
+
+#ifndef SOCK_SNDBUF_SIZE
+#define SOCK_SNDBUF_SIZE             0
+#endif
+
+#ifndef SOCK_LINGER
+#define SOCK_LINGER                  0
+#endif
+
+#ifndef SOCK_NO_DELAY 
+#define SOCK_NO_DELAY                1
+#endif
 
 //
 // Abstract socket interface
