@@ -37,6 +37,7 @@ enum SubSqlTokens {
     tkn_help,
     tkn_http, 
     tkn_import,
+    tkn_incremental,
     tkn_index,
     tkn_int1,
     tkn_int2,
@@ -53,6 +54,7 @@ enum SubSqlTokens {
     tkn_real8,
     tkn_rectangle,
     tkn_reference,
+    tkn_reorder,
     tkn_restore,
     tkn_rename,
     tkn_rollback,
