@@ -21,7 +21,7 @@ class Record {
     db_int8       intKey;
     char_t const* strKey;
 
-    TYPE_DESCRIPTOR((KEY(intKey, INDEXED), KEY(strKey, INDEXED)));
+    TYPE_DESCRIPTOR((KEY(intKey, INDEXED), KEY(strKey, CASE_INSENSITIVE|INDEXED)));
 };
 
 REGISTER(Record);
@@ -41,7 +41,7 @@ int __cdecl main(int argc, char* argv[])
 
     dbDatabase db(dbDatabase::dbAllAccess, 4*1024); // 32Mb page pool
     if (db.open(_T("testindex.dbs"))) {
-        dbQuery q1, q2, q3, q4, q5;
+        dbQuery q1, q2, q3, q4, q5, q6;
         dbCursor<Record> cursor1;
         dbCursor<Record> cursor2;
         q1 = _T("intKey="),intKey;
@@ -53,6 +53,7 @@ int __cdecl main(int argc, char* argv[])
 #endif
         q4 = _T("order by intKey");
         q5 = _T("order by strKey");
+        q6 = _T("lower(strKey) like"),strKey;
         db_nat8 key = 1999;
         unsigned start = GET_TIME();
         for (i = 0; i < nRecords; i++) {
@@ -84,6 +85,17 @@ int __cdecl main(int argc, char* argv[])
         printf("Elapsed time for %d index searches: %d\n",
                nRecords*2, int(GET_TIME() - start));
 
+        start = GET_TIME();
+        key = 1999;
+        for (i = 0; i < nRecords; i++) {
+            key = (3141592621u*key + 2718281829u) % 1000000007u;
+            SPRINTF(SPRINTF_BUFFER(strKey), T_INT8_FORMAT _T(".%%"), key);
+            n = cursor2.select(q6);
+            assert(n == 1);
+        }
+        printf("Elapsed time for %d LIKE queries: %d\n",
+               nRecords, int(GET_TIME() - start));
+
         start = GET_TIME();
         n = cursor1.select(q4);
         assert(n == nRecords);
