@@ -142,7 +142,7 @@ int __cdecl main(int argc, char* argv[])
     if (db.open("testsync.dbs", commitDelay)) {
         db.setConcurrency(0);
         for (i = 0; i < nThreads; i++) {
-            thread[i].create(doTest, (void*)id++);
+            thread[i].create(doTest, (void*)size_t(id++));
         }
         for (i = 0; i < nThreads; i++) {
             thread[i].join();
