@@ -14,6 +14,7 @@
 
 #include "stdtp.h"
 #include "w32sock.h"
+#include "sync.h"
 
 BEGIN_GIGABASE_NAMESPACE
 
@@ -21,15 +22,46 @@ BEGIN_GIGABASE_NAMESPACE
 #define MILLISECOND           1000
 
 static HANDLE WatchDogMutex;
-
-#if !defined(ASM_CPUID_NOT_SUPPORTED) && (defined(__MINGW32__) || (defined(_MSC_VER) && _MSC_VER < 1200))
-#define ASM_CPUID_NOT_SUPPORTED
+#ifdef L64
 #endif
 
-#ifdef ASM_CPUID_NOT_SUPPORTED
-static CRITICAL_SECTION cs;
+#if (defined(_MSC_VER) && _MSC_VER <= 1310) || defined(__GNUC__) || defined(__BORLANDC__)
+#define MEMORY_BARRIER_NOT_DEFINED true
+#endif 
+
+#ifdef MEMORY_BARRIER_NOT_DEFINED
+
+#ifdef __GNUC__
+
+#if (__GNUC__ > 4) || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1)
+#define MemoryBarrier() __sync_synchronize()
+#else
+inline void MemoryBarrier() { 
+    asm volatile("mfence":::"memory");
+}
 #endif
 
+#else
+
+#if defined(_M_AMD64)
+#define MemoryBarrier __faststorefence
+#elif defined(_M_IA64)
+#define MemoryBarrier __mf
+#elif (defined(_M_IX86) || defined(_M_X64)) && !defined(__BORLANDC__)
+inline void MemoryBarrier() {
+    LONG Barrier;
+    __asm {
+        xchg Barrier, eax
+    }
+}
+#else
+static dbMutex barrierMutex;
+inline void MemoryBarrier() { dbCriticalSection cs(barrierMutex); }
+#endif
+#endif
+#endif
+
+
 class win_socket_library {
   public:
     SYSTEM_INFO sinfo;
@@ -46,15 +78,9 @@ class win_socket_library {
         WatchDogMutex = CreateMutex(NULL, TRUE, NULL);
 
         GetSystemInfo(&sinfo);  
-#ifdef ASM_CPUID_NOT_SUPPORTED
-        InitializeCriticalSection(&cs);
-#endif
     }
     ~win_socket_library() {
-#ifdef ASM_CPUID_NOT_SUPPORTED
-        DeleteCriticalSection(&cs);
-#endif
-        //      WSACleanup();
+        // WSACleanup();
     }
 };
 
@@ -83,23 +109,6 @@ dbNullSecurityDesciptor dbNullSecurityDesciptor::instance;
 
 static win_socket_library ws32_lib;
 
-#ifdef __BORLANDC__
-static
-#else
-inline 
-#endif
-void serialize() { 
-#ifndef ASM_CPUID_NOT_SUPPORTED
-    if (ws32_lib.sinfo.wProcessorLevel >= 5) { // Pemtium or higher 
-        __asm CPUID;
-    }
-#else
-    EnterCriticalSection(&cs);
-    LeaveCriticalSection(&cs);
-#endif
-}
-
-
 bool win_socket::open(int listen_queue_size)
 {
     unsigned short port;
@@ -123,7 +132,7 @@ bool win_socket::open(int listen_queue_size)
     }
     struct sockaddr_in insock;
     insock.sin_family = AF_INET;
-    if (*hostname && stricmp(hostname, "localhost") != 0) {
+    if (*hostname && _stricmp(hostname, "localhost") != 0) {
         struct hostent* hp;  // entry in hosts table
         if ((hp = gethostbyname(hostname)) == NULL
             || hp->h_addrtype != AF_INET)
@@ -191,10 +200,10 @@ void win_socket::get_error_text(char_t* buf, size_t buf_size)
                             errcode,
                             0,
                             buf,
-                            buf_size-1,
+                            (DWORD)(buf_size-1),
                             NULL);
         if (len == 0) {
-            SPRINTF(msgbuf, _T("unknown error code %u"), errcode);
+            SPRINTF(SPRINTF_BUFFER(msgbuf), _T("unknown error code %u"), errcode);
             msg = msgbuf;
         } else { 
             return;
@@ -217,12 +226,14 @@ socket_t* win_socket::accept()
         errcode = WSAGetLastError();
         return NULL;
     } else {
+#ifdef USE_SO_LINGER
         static struct linger l = {1, LINGER_TIME};
         if (setsockopt(new_sock, SOL_SOCKET, SO_LINGER, (char*)&l, sizeof l) != 0) {
             errcode = invalid_access_mode;
             closesocket(new_sock);
             return NULL;
         }
+#endif
         int enabled = 1;
         if (setsockopt(new_sock, IPPROTO_TCP, TCP_NODELAY, (char*)&enabled,
                        sizeof enabled) != 0)
@@ -296,13 +307,21 @@ bool win_socket::connect(int max_attempts, time_t timeout)
                     closesocket(s);
                     return false;
                 }
+#ifdef USE_SO_LINGER
+                static struct linger l = {1, LINGER_TIME};
+                if (setsockopt(s, SOL_SOCKET, SO_LINGER, (char*)&l, sizeof l) != 0) {
+                    errcode = invalid_access_mode;
+                    closesocket(s);
+                    return NULL;
+                }
+#endif
                 errcode = ok;
                 state = ss_open;
                 return true;
             }
         }
         if (--max_attempts > 0) {
-            Sleep(timeout*MILLISECOND);
+            Sleep((DWORD)(timeout*MILLISECOND));
         } else {
             errcode = connection_failed;
             return false;
@@ -330,20 +349,20 @@ int win_socket::read(void* buf, size_t min_size, size_t max_size,
             struct timeval tm;
             FD_ZERO(&events);
             FD_SET(s, &events);
-            tm.tv_sec = timeout;
+            tm.tv_sec = (long)timeout;
             tm.tv_usec = 0;
-            rc = select(s+1, &events, NULL, NULL, &tm);
+            rc = select((int)s+1, &events, NULL, NULL, &tm);
             if (rc < 0) {
                 errcode = WSAGetLastError();
                 return -1;
             }
             if (rc == 0) {
-                return size;
+                return (int)size;
             }
             time_t now = time(NULL);
             timeout = start + timeout >= now ? timeout + start - now : 0;
         }
-        rc = recv(s, (char*)buf + size, max_size - size, 0);
+        rc = recv(s, (char*)buf + size, (int)(max_size - size), 0);
         if (rc < 0) {
             errcode = WSAGetLastError();
             return -1;
@@ -366,7 +385,7 @@ bool win_socket::write(void const* buf, size_t size)
     }
 
     do {
-        int rc = send(s, (char*)buf, size, 0);
+        int rc = send(s, (char*)buf, (int)size, 0);
         if (rc < 0) {
             errcode = WSAGetLastError();
             return false;
@@ -490,7 +509,7 @@ socket_t* socket_t::connect(char const* address,
             && ((port = strchr(address, ':')) == NULL
                 || ((hostname_len = port - address) == 9
                     && strncmp(address, "localhost", hostname_len) == 0)
-                || (gethostname(hostname, sizeof hostname) != 0
+                || (gethostname(hostname, sizeof hostname) == 0
                     && strlen(hostname) == hostname_len
                     && strncmp(address, hostname, hostname_len) == 0))))
      {
@@ -526,7 +545,7 @@ int local_win_socket::read(void* buf, size_t min_size, size_t max_size,
     }
     while (size < min_size && state == ss_open) {
         RcvBuf->RcvWaitFlag = true;
-        serialize();
+        MemoryBarrier();
         size_t begin = RcvBuf->DataBeg;
         size_t end = RcvBuf->DataEnd;
         size_t rcv_size = (begin <= end)
@@ -544,7 +563,8 @@ int local_win_socket::read(void* buf, size_t min_size, size_t max_size,
                 size += rcv_size;
                 max_size -= rcv_size;
             }
-            RcvBuf->DataBeg = (begin == sizeof(RcvBuf->Data)) ? 0 : begin;
+            RcvBuf->DataBeg = (begin == sizeof(RcvBuf->Data)) ? 0 : (int)begin;
+            MemoryBarrier();
             if (RcvBuf->SndWaitFlag) {
                 SetEvent(Signal[RTR]);
             }
@@ -552,14 +572,14 @@ int local_win_socket::read(void* buf, size_t min_size, size_t max_size,
             HANDLE h[2];
             h[0] = Signal[RD];
             h[1] = Mutex;
-            int rc = WaitForMultipleObjects(2, h, false, timeout);
+            int rc = WaitForMultipleObjects(2, h, false, (DWORD)timeout);
             RcvBuf->RcvWaitFlag = false;
             if (rc != WAIT_OBJECT_0) {
                 if (rc == WAIT_OBJECT_0+1 || rc == WAIT_ABANDONED+1) {
                     Error = broken_pipe;
                     ReleaseMutex(Mutex);
                 } else if (rc == WAIT_TIMEOUT) {
-                    return size;
+                    return (int)size;
                 } else {
                     Error = GetLastError();
                 }
@@ -582,7 +602,7 @@ bool local_win_socket::write(const void* buf, size_t size)
     Error = ok;
     while (size > 0 && state == ss_open) {
         SndBuf->SndWaitFlag = true;
-        serialize();
+        MemoryBarrier();
         size_t begin = SndBuf->DataBeg;
         size_t end = SndBuf->DataEnd;
         size_t snd_size = (begin <= end)
@@ -600,7 +620,8 @@ bool local_win_socket::write(const void* buf, size_t size)
                 src += snd_size;
                 size -= snd_size;
             }
-            SndBuf->DataEnd = (end == sizeof(SndBuf->Data)) ? 0 : end;
+            SndBuf->DataEnd = (end == sizeof(SndBuf->Data)) ? 0 : (int)end;
+            MemoryBarrier();
             if (SndBuf->RcvWaitFlag) {
                 SetEvent(Signal[TD]);
             }
@@ -609,7 +630,7 @@ bool local_win_socket::write(const void* buf, size_t size)
             h[0] = Signal[RTT];
             h[1] = Mutex;
             int rc = WaitForMultipleObjects(2, h, false, INFINITE);
-            RcvBuf->SndWaitFlag = false;
+            SndBuf->SndWaitFlag = false;
             if (rc != WAIT_OBJECT_0) {
                 if (rc == WAIT_OBJECT_0+1 || rc == WAIT_ABANDONED+1) {
                     Error = broken_pipe;
@@ -844,7 +865,7 @@ void local_win_socket::get_error_text(char_t* buf, size_t buf_size)
                       Error,
                       0,
                       buf,
-                      buf_size-1,
+                      (DWORD)(buf_size-1),
                       NULL);
     }
     buf[buf_size-1] = '\0';
@@ -901,7 +922,7 @@ bool local_win_socket::connect(int max_attempts, time_t timeout)
     state = ss_shutdown;
     Mutex = NULL;
 
-    rc = WaitForSingleObject(Signal[RTT],timeout*max_attempts*MILLISECOND);
+    rc = WaitForSingleObject(Signal[RTT], (DWORD)(timeout*max_attempts*MILLISECOND));
     if (rc != WAIT_OBJECT_0) {
         error_code = (rc == WAIT_TIMEOUT) ? (int)timeout_expired : (int)GetLastError();
         close();
