@@ -3,14 +3,19 @@
 
 #include <winsock.h>
 
+#if _WIN32_WCE >= 0x400
+#include <time.h>
+#include <assert.h>
+#else
 #include "wince_time.h"
-
 #define assert(x) 
-#define stricmp   _stricmp
-#define getenv(x) 0
+#endif
 
+#ifndef TLS_OUT_OF_INDEXES
+#define TLS_OUT_OF_INDEXES 0xFFFFFFFF
+#endif
+
+#define getenv(x) 0
 int abort();
 
-#define ASM_CPUID_NOT_SUPPORTED 1
-
 #endif
